import os
import importlib.util
import re

def extract_function(urls_content, url):
    # urls = ast.literal_eval(urls_content)
    match = re.search(r"urls = (\[.+\])", urls_content)
    if not match:
        return False

    urls = eval(match.group(1))
    for u in urls:
        if u[0] == url:
            return u[1]
    return False

def change_path_format(path):
    components = path.split(os.path.sep)

    dot_string = ".".join(components)
    return dot_string

def get_view(url, project_path):
    urls_path = os.path.join(project_path, "urls.py")

    if not os.path.exists(urls_path):
        return False

    with open(urls_path, "r") as f:
        contents = f.read()

    class_name = extract_function(contents, url)

    views_path = os.path.join(project_path, "views.py")
    spec = importlib.util.spec_from_file_location("views", str(views_path))
    views_module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(views_module)

    view_func = getattr(views_module, class_name)

    return view_func()

def handle_request(project_name, path, method, form):
    # Handle the root route
    response_body = get_view(path, project_name)

    if not response_body:
        return "HTTP/1.1 404 Not Found\nContent-Type: text/plain\nContent-Length: 9\n\nNot Found"

    return "HTTP/1.1 200 OK\nContent-Type: text/html\nContent-Length: {}\n\n{}".format(len(response_body), response_body)



import os
import json

celestis_content = r'''

import socket
from celestis.controller import request as rq
from error import *
import os
import json
import signal

def parse_request(request):
    if request == "":
        return "GET", "/", ""
    lines = request.split("\n")
    method, path, headers = lines[0].split(" ")
    headers = dict(line.split(": ") for line in lines[1:-2])
    return method, path, headers

def parse_form(headers):
    if "Content-Type" not in headers:
        return {}
    if headers["Content-Type"] != "application/x-www-form-urlencoded":
        return {}
    return dict(pair.split("=") for pair in headers["Content-Length"].split("&"))


sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.bind(("localhost", 8080))
sock.listen(8080)

def handle_sigint(signal, frame):
    if conn:
        conn.close()
    sock.close()

    print("Your server has been closed. Thanks for connecting!")

    exit(0)

signal.signal(signal.SIGINT, handle_sigint)

conn = False

print("Celestis server is listening at port 8080...\nTo close the server, visit http://localhost:8080 and then press Ctrl-C on the terminal")

while True:
    conn, addr = sock.accept()
    request = conn.recv(1024).decode("utf-8")

    method, path, headers = parse_request(request)
    form = parse_form(headers)
    response = rq.handle_request(os.getcwd(), path, method, form)
    conn.sendall(response.encode("utf-8"))
    conn.close()
'''

urls_page_content = '''
# urls for the {} project\n
urls = [("/", "home")]
'''

views_page_content = '''
# views for {} project\n
def home():\n
    return "<h1>{} says hi!</h1>"
'''

def create_app(name):
    # Create the app folder
    app_folder = name
    os.makedirs(app_folder, exist_ok=True)
    
    # Create the views.py file
    views_path = os.path.join(app_folder, "views.py")
    with open(views_path, "w") as f:
        f.write(views_page_content.format(name))
    
    # Create the urls.py file
    urls_path = os.path.join(app_folder, "urls.py")
    with open(urls_path, "w") as f:
        f.write(urls_page_content.format(name))

    celestis_path = os.path.join(app_folder, "server.py")
    with open(celestis_path, "w") as f:
        f.write(celestis_content)
    
    meta_path = os.path.join(app_folder, "meta.json")
    with open(meta_path, "w") as f:
        data = {"project": name}
        json.dump(data, f)
    


import os

def contextualize(html, variables):
    # Fix with a templating engine

    return html

def render_template(file_path, **context):
    with open(file_path) as f:
        template = f.read()
        if os.path.exists(file_path) and not template:
            template = f"<p>The HTML file at {file_path} was empty</p>"

    template = contextualize(template, context)

    return template
    # Templating using Jinja and context.items()
        


from setuptools import setup

setup(
    name='celestis',
    version='1.7',
    description='A simple backend framework built using python',
    author="Aryaan Hegde",
    author_email="aryhegde@gmail.com",
    packages=["celestis.controller", "celestis.view"],
    package_dir={"celestis.controller": "controller", "celestis.view": "view"},
    py_modules=['celestis', 'command', 'create_files', 'error', 'render', 'requests'],
    include_package_data=True,
    install_requires=['requests', 'click'],
    entry_points={
        'console_scripts': [
            'celestis=command:celestis',
        ],
    },
)